# -*- coding: utf-8 -*-
# Copyright 2014 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
from __future__ import absolute_import, unicode_literals
import logging
import os

import requests
from cliff.command import Command

from click_toolbelt.constants import MYAPPS_API_ROOT_URL


class Info(Command):
    log = logging.getLogger(__name__)
    topics = ['version', 'department', 'license', 'country']

    def get_parser(self, prog_name):
        parser = super(Info, self).get_parser(prog_name)
        parser.add_argument('topic', nargs='?', choices=self.topics)
        return parser

    def get_info(self):
        result = {'success': False, 'errors': [], 'data': None}
        api_endpoint = os.environ.get(
            'MYAPPS_API_ROOT_URL', MYAPPS_API_ROOT_URL)
        response = requests.get(api_endpoint)
        if response.ok:
            result['success'] = True
            result['data'] = response.json()
        else:
            result['errors'] = [response.text]
        return result

    def get_version_info(self, data):
        return data.get('version')

    def get_department_info(self, data):
        return data.get('/dev/api/click-app/', {}).get('department')

    def get_license_info(self, data):
        return data.get('/dev/api/click-app/', {}).get('license')

    def get_country_info(self, data):
        return data.get('/dev/api/click-app/', {}).get('country')

    def get_topic_data(self, api_data, topic):
        method = getattr(self, 'get_{}_info'.format(topic))
        return method(api_data)

    def take_action(self, parsed_args):
        topic = parsed_args.topic
        result = self.get_info()

        if not result.get('success', False):
            self.log.info(
                'Could not get information. An error ocurred:\n\n%s\n\n',
                '\n'.join(result['errors']))
            return

        api_data = result['data']
        self.log.info('API info:')

        if topic:
            topics = [topic]
        else:
            topics = self.topics

        for topic in topics:
            data = self.get_topic_data(api_data, topic)
            if data:
                self.log.info('  %s: %s', topic, data)



# Copyright 2013 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
#!/usr/bin/env python
from __future__ import absolute_import, unicode_literals
import sys

from cliff.app import App
from cliff.commandmanager import CommandManager

from click_toolbelt import __namespace__, __version__


class ClickToolbelt(App):
    def __init__(self):
        super(ClickToolbelt, self).__init__(
            description='Tools for working with click packages',
            version=__version__,
            command_manager=CommandManager(__namespace__))


def main(args=None):
    if args is None:
        args = sys.argv[1:]

    toolbelt = ClickToolbelt()
    return toolbelt.run(args)


if __name__ == '__main__':
    args = sys.argv[1:]
    sys.exit(main(args))



# Copyright 2014 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
from __future__ import absolute_import, unicode_literals
import json
import logging
import os

from click_toolbelt.common import ClickToolbeltCommand, retry
from click_toolbelt.compat import open, urljoin
from click_toolbelt.constants import (
    CLICK_UPDOWN_UPLOAD_URL,
    MYAPPS_API_ROOT_URL,
    SCAN_STATUS_POLL_DELAY,
    SCAN_STATUS_POLL_RETRIES,
)


def is_scan_completed(response):
    if response.ok:
        return response.json().get('completed', False)
    return False


class Register(ClickToolbeltCommand):
    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super(Register, self).get_parser(prog_name)
        parser.add_argument('binary_filename')
        source_group = parser.add_mutually_exclusive_group()
        source_group.add_argument('source_filename', nargs='?')
        source_group.add_argument('--source', metavar='source_filename',
                                  dest='source_filename')
        metadata_group = parser.add_mutually_exclusive_group()
        metadata_group.add_argument('metadata_filename', nargs='?')
        metadata_group.add_argument('--metadata', metavar='metadata_filename',
                                    dest='metadata_filename')
        return parser

    def upload_files(self, binary_filename, source_filename=None):
        updown_url = os.environ.get('CLICK_UPDOWN_UPLOAD_URL',
                                    CLICK_UPDOWN_UPLOAD_URL)
        unscanned_upload_url = urljoin(updown_url, 'unscanned-upload/')
        files = {'binary': open(binary_filename, 'rb')}
        if source_filename is not None:
            files['source'] = open(source_filename, 'rb')

        result = {'success': False, 'errors': []}

        session = self.get_oauth_session()
        if session is None:
            result['errors'] = ['No valid credentials found.']
            return result

        try:
            response = session.post(
                unscanned_upload_url,
                files=files)
            if response.ok:
                response_data = response.json()
                result.update({
                    'success': response_data.get('successful', True),
                    'upload_id': response_data['upload_id'],
                    'binary_filesize': os.path.getsize(binary_filename),
                    'source_uploaded': 'source' in files,
                })
            else:
                self.log.error(
                    'There was an error uploading the click package.\n'
                    'Reason: %s\n'
                    'Text: %s',
                    response.reason, response.text)
                result['errors'] = [response.text]
        except Exception as err:
            self.log.exception(
                'An unexpected error was found while uploading files.')
            result['errors'] = [str(err)]
        finally:
            # make sure to close any open files used for request
            for fd in files.values():
                fd.close()

        return result

    def get_data(self, upload_data, metadata=None):
        data = {
            'updown_id': upload_data['upload_id'],
            'binary_filesize': upload_data['binary_filesize'],
            'source_uploaded': upload_data['source_uploaded'],
        }
        data.update({
            key: value
            for (key, value) in metadata.items()
            if key not in (
                # make sure not to override upload_id, binary_filesize and
                # source_uploaded
                'upload_id', 'binary_filesize', 'source_uploaded',
                # skip files as they will be added to the files argument
                'icon_256', 'icon', 'screenshots',
            )
        })
        return data

    def get_files(self, metadata=None):
        files = []

        icon = metadata.get('icon', metadata.get('icon_256', ''))
        if icon:
            icon_file = open(icon, 'rb')
            files.append(('icon_256', icon_file))

        screenshots = metadata.get('screenshots', [])
        for screenshot in screenshots:
            screenshot_file = open(screenshot, 'rb')
            files.append(('screenshots', screenshot_file))

        return files

    def register_app(self, upload_data, metadata=None):
        myapps_api_url = os.environ.get('MYAPPS_API_ROOT_URL',
                                        MYAPPS_API_ROOT_URL)
        register_url = urljoin(myapps_api_url, 'click-app/')

        result = {'success': False, 'errors': [], 'application_url': ''}

        session = self.get_oauth_session()
        if session is None:
            result['errors'] = ['No valid credentials found.']
            return result

        if metadata is None:
            metadata = {}

        try:
            data = self.get_data(upload_data, metadata=metadata)
            files = self.get_files(metadata=metadata)

            response = session.post(register_url, data=data, files=files)
            if response.ok:
                status_url = response.json()['status_url']
                self.log.info('Package submitted to %s', register_url)
                self.log.info('Checking package status...')
                completed, data = self._get_scan_data(session, status_url)
                if completed:
                    self.log.info('Package scan completed.')
                    message = data.get('message', '')
                    if not message:
                        result['success'] = True
                    else:
                        result['errors'] = [message]
                else:
                    result['errors'] = [
                        'Package scan took too long.',
                        'Please check the status later at: %s.' % status_url,
                    ]
                result['application_url'] = data.get('application_url', '')
            else:
                self.log.error(
                    'There was an error registering the click application.\n'
                    'Reason: %s\n'
                    'Text: %s',
                    response.reason, response.text)
                result['errors'] = [response.text]
        except Exception as err:
            self.log.exception(
                'There was an error registering the application.')
            result['errors'] = [str(err)]
        finally:
            # make sure to close any open files used for request
            for fname, fd in files:
                fd.close()

        return result

    def _get_scan_data(self, session, status_url):
        # initial retry after 10 seconds
        # linear backoff after that
        # abort after 5 retries
        @retry(terminator=is_scan_completed,
               retries=SCAN_STATUS_POLL_RETRIES,
               delay=SCAN_STATUS_POLL_DELAY,
               backoff=1, logger=self.log)
        def get_status():
            return session.get(status_url)

        response, aborted = get_status()

        completed = False
        data = {}
        if not aborted:
            completed = is_scan_completed(response)
            data = response.json()
        return completed, data

    def take_action(self, parsed_args):
        self.log.info('Running register command...')

        binary_filename = parsed_args.binary_filename
        source_filename = parsed_args.source_filename
        metadata_filename = parsed_args.metadata_filename

        # do the request
        self.log.info('Uploading files...')
        data = self.upload_files(
            binary_filename, source_filename=source_filename)
        success = data.get('success', False)
        errors = data.get('errors', [])

        if not success:
            self.log.info('Upload failed:\n\n%s\n', '\n'.join(errors))
            return

        if metadata_filename:
            with open(metadata_filename, 'r') as metadata_file:
                # file is automatically closed by context manager
                metadata = json.load(metadata_file)
        else:
            metadata = {}

        self.log.info('Registering application...')
        result = self.register_app(data, metadata=metadata)
        success = result.get('success', False)
        errors = result.get('errors', [])
        app_url = result.get('application_url', '')

        if success:
            self.log.info('Application registered successfully.')
        else:
            self.log.info('Registration did not complete.')

        if errors:
            self.log.info('Some errors were detected:\n\n%s\n\n',
                          '\n'.join(errors))

        if app_url:
            self.log.info('Please check out the application at: %s.\n',
                          app_url)



# Copyright 2014 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
from __future__ import absolute_import, unicode_literals
import os
import time
from functools import wraps

from cliff.command import Command
from requests_oauthlib import OAuth1Session
from xdg.BaseDirectory import load_first_config

from click_toolbelt import __namespace__
from click_toolbelt.compat import ConfigParser, urlparse
from click_toolbelt.constants import (
    CLICK_TOOLBELT_PROJECT_NAME,
    UBUNTU_SSO_API_ROOT_URL,
)


class ClickToolbeltCommand(Command):

    def get_config(self):
        config_dir = load_first_config(CLICK_TOOLBELT_PROJECT_NAME)
        filename = os.path.join(config_dir, "%s.cfg" % __namespace__)
        parser = ConfigParser()
        if os.path.exists(filename):
            parser.read(filename)

        api_endpoint = os.environ.get(
            'UBUNTU_SSO_API_ROOT_URL', UBUNTU_SSO_API_ROOT_URL)
        location = urlparse(api_endpoint).netloc

        config = {}
        if parser.has_section(location):
            config.update(dict(parser.items(location)))
        return config

    def get_oauth_session(self):
        config = self.get_config()
        try:
            session = OAuth1Session(
                config['consumer_key'],
                client_secret=config['consumer_secret'],
                resource_owner_key=config['token_key'],
                resource_owner_secret=config['token_secret'],
                signature_method='PLAINTEXT',
            )
        except KeyError:
            session = None
        return session

    def take_action(self, parsed_args):
        pass


def retry(terminator=None, retries=3, delay=3, backoff=2, logger=None):
    """Decorate a function to automatically retry calling it on failure.

    Arguments:
        - terminator: this should be a callable that returns a boolean;
          it is used to determine if the function call was successful
          and the retry loop should be stopped
        - retries: an integer specifying the maximum number of retries
        - delay: initial number of seconds to wait for the first retry
        - backoff: exponential factor to use to adapt the delay between
          subsequent retries
        - logger: logging.Logger instance to use for logging

    The decorated function will return as soon as any of the following
    conditions are met:

        1. terminator evaluates function output as True
        2. there are no more retries left

    If the terminator callable is not provided, the function will be called
    exactly once and will not be retried.

    """
    def decorated(func):
        if retries != int(retries) or retries < 0:
            raise ValueError(
                'retries value must be a positive integer or zero')
        if delay < 0:
            raise ValueError('delay value must be positive')

        if backoff != int(backoff) or backoff < 1:
            raise ValueError('backoff value must be a positive integer')

        @wraps(func)
        def wrapped(*args, **kwargs):
            retries_left, current_delay = retries, delay

            result = func(*args, **kwargs)
            if terminator is not None:
                while not terminator(result) and retries_left > 0:
                    msg = "... retrying in %d seconds" % current_delay
                    if logger:
                        logger.warning(msg)

                    # sleep
                    time.sleep(current_delay)
                    retries_left -= 1
                    current_delay *= backoff

                    # retry
                    result = func(*args, **kwargs)
            return result, retries_left == 0

        return wrapped
    return decorated



# Copyright 2013 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
from __future__ import absolute_import, unicode_literals
import sys


PY2 = sys.version_info[0] == 2
if PY2:
    from __builtin__ import open
    from ConfigParser import ConfigParser
    from urlparse import urljoin
    from urlparse import urlparse
else:
    from builtins import open
    from configparser import ConfigParser
    from urllib.parse import urljoin
    from urllib.parse import urlparse



# Copyright 2013, 2014 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
from __future__ import absolute_import, unicode_literals


CLICK_TOOLBELT_PROJECT_NAME = 'click-toolbelt'
CLICK_UPDOWN_UPLOAD_URL = 'https://upload.apps.ubuntu.com/upload/'
MYAPPS_API_ROOT_URL = 'https://myapps.developer.ubuntu.com/dev/api/'
UBUNTU_SSO_API_ROOT_URL = 'https://login.ubuntu.com/api/v2/'
SCAN_STATUS_POLL_DELAY = 10
SCAN_STATUS_POLL_RETRIES = 5



# Copyright 2013 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
from __future__ import absolute_import, unicode_literals
import logging
import os

import requests

from click_toolbelt.common import ClickToolbeltCommand
from click_toolbelt.compat import open, urljoin
from click_toolbelt.constants import (
    CLICK_UPDOWN_UPLOAD_URL,
    MYAPPS_API_ROOT_URL,
)


class Upload(ClickToolbeltCommand):
    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super(Upload, self).get_parser(prog_name)
        parser.add_argument('application_name')
        parser.add_argument('version')
        parser.add_argument('filename')
        parser.add_argument('changelog')
        parser.add_argument('framework', nargs='?')
        parser.add_argument(
            'arch', nargs='*', default='all',
            choices=['armhf', 'i386', 'amd64', 'all'])
        return parser

    def get_upload_data(
            self, name, version, changelog, architectures, framework):
        myapps_url = os.environ.get('MYAPPS_API_ROOT_URL', MYAPPS_API_ROOT_URL)
        myapps_url = urljoin(myapps_url, 'click-upload/%s/' % name)

        result = {'success': False, 'errors': []}

        session = self.get_oauth_session()
        if session is None:
            result['errors'] = ['No valid credentials found.']
            return result

        try:
            response = session.post(
                myapps_url,
                data={'version': version, 'changelog': changelog,
                      'architectures': architectures, 'framework': framework})
            if response.ok:
                result.update(response.json())
            else:
                result['errors'] = [response.text]
        except Exception as err:
            self.log.exception("There was an error getting upload data for "
                               "application '%s' version '%s'.", name, version)
            result['errors'] = [str(err)]
        return result

    def mark_uploaded(self, uploaded_callback_url, name, version, binary_size):
        result = {'success': False, 'errors': []}

        data = {
            'version': version,
            'binary': binary_size,
        }

        session = self.get_oauth_session()
        if session is None:
            result['errors'] = ['No valid credentials found.']
            return result

        try:
            response = session.post(uploaded_callback_url, data=data)
            if response.ok:
                result.update(response.json())
            else:
                result['errors'] = [response.text]
        except Exception as err:
            self.log.exception("There was an error marking application '%s' "
                               "version '%s' as uploaded.", name, version)
            result['errors'] = [str(err)]
        return result

    def take_action(self, parsed_args):
        self.log.info('Running upload command...')

        application_name = parsed_args.application_name
        version = parsed_args.version
        filename = parsed_args.filename
        changelog = parsed_args.changelog
        architectures = parsed_args.arch
        framework = parsed_args.framework

        data = self.get_upload_data(
            application_name, version, changelog, architectures, framework)
        if not data.get('success', False):
            errors = data.get('errors', ['Unknown error'])
            self.log.error(
                'Error verifying upload request. Uploading not possible.\n'
                'Reason: %s', '\n'.join(errors))
            return

        uploaded_callback_url = data.pop('uploaded_callback_url', None)
        files = {'binary': open(filename, 'rb')}

        # do the request
        upload_url = os.environ.get('CLICK_UPDOWN_UPLOAD_URL',
                                    CLICK_UPDOWN_UPLOAD_URL)

        success = False
        response = None
        try:
            response = requests.post(upload_url, data=data, files=files)
            if response.ok:
                data = response.json()
                success = data.get('successful', False)
        except Exception:
            self.log.exception(
                'There was an error uploading the click package.')
            return

        if success:
            binary_size = os.path.getsize(filename)
            data = self.mark_uploaded(uploaded_callback_url, application_name,
                                      version, binary_size)
            if data.get('success', False):
                self.log.info('Click Package upload finished successfully.')
            else:
                errors = data.get('errors', ['Unknown error'])
                self.log.error(
                    'There was an error after uploading the click package.\n'
                    'Reason: %s', '\n'.join(errors))
        else:
            self.log.error(
                'There was an error uploading the click package.\n'
                'Reason: %s\n'
                'Text: %s',
                response.reason, response.text)



# -*- coding: utf-8 -*-
# Copyright 2013 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
from __future__ import absolute_import, unicode_literals
import json
import os
import shutil
import tempfile
from collections import namedtuple
from unittest import TestCase
from xdg.BaseDirectory import save_config_path

from mock import patch
from requests import Response

from click_toolbelt import __namespace__
from click_toolbelt.compat import ConfigParser
from click_toolbelt.login import (
    CLICK_TOOLBELT_PROJECT_NAME,
    UBUNTU_SSO_API_ROOT_URL,
    ApiException,
    Login,
)


class LoginCommandTestCase(TestCase):

    def setUp(self):
        super(LoginCommandTestCase, self).setUp()
        app = None
        args = None
        self.command = Login(app, args)
        self.email = 'user@domain.com'
        self.password = 'password'
        parsed_args = namedtuple('parsed_args', 'email, password, otp')
        self.parsed_args = parsed_args(self.email, self.password, '')

        # setup patches
        mock_environ = {
            'UBUNTU_SSO_API_ROOT_URL': UBUNTU_SSO_API_ROOT_URL,
        }
        patcher = patch('click_toolbelt.login.os.environ', mock_environ)
        patcher.start()
        self.addCleanup(patcher.stop)

        patcher = patch('click_toolbelt.login.Login.log')
        self.mock_log = patcher.start()
        self.addCleanup(patcher.stop)

    def test_parser(self):
        parser = self.command.get_parser(__namespace__)
        for i, (name, required) in enumerate(
                [('email', True), ('password', True), ('otp', False)]):
            # argument 0 is builtin --help
            # start comparing from first extra argument
            self.assertEqual(parser._actions[i + 1].dest, name)
            self.assertEqual(parser._actions[i + 1].required, required)

    def setup_config_dir(self, include_click_dir=True):
        config_dir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, config_dir)
        patcher = patch('xdg.BaseDirectory.xdg_config_home',
                        config_dir)
        patcher.start()
        self.addCleanup(patcher.stop)

        if include_click_dir:
            save_config_path(CLICK_TOOLBELT_PROJECT_NAME)
        return config_dir

    def test_save_config(self):
        config_dir = self.setup_config_dir()
        data = {'foo': '1', 'bar': '2', 'baz': '3'}
        expected_filename = os.path.join(config_dir,
                                         CLICK_TOOLBELT_PROJECT_NAME,
                                         "{}.cfg".format(__namespace__))

        self.command.save_config(data)

        cfg = ConfigParser()
        cfg.read(expected_filename)
        self.assertTrue(cfg.has_section('login.ubuntu.com'))
        self.assertEqual(dict(cfg.items('login.ubuntu.com')), data)

    def test_save_config_with_existing_data(self):
        config_dir = self.setup_config_dir()
        expected_filename = os.path.join(config_dir,
                                         CLICK_TOOLBELT_PROJECT_NAME,
                                         "{}.cfg".format(__namespace__))
        cfg = ConfigParser()
        cfg.add_section('login.ubuntu.com')
        cfg.set('login.ubuntu.com', 'foo', 'foo')
        cfg.write(open(expected_filename, 'w'))
        data = {'foo': '1', 'bar': '2', 'baz': '3'}

        self.command.save_config(data)

        cfg.read(expected_filename)
        self.assertTrue(cfg.has_section('login.ubuntu.com'))
        self.assertEqual(dict(cfg.items('login.ubuntu.com')), data)

    def test_config_dir_created_if_necessary(self):
        config_dir = self.setup_config_dir(include_click_dir=False)
        data = {'foo': '1', 'bar': '2', 'baz': '3'}
        expected_filename = os.path.join(config_dir,
                                         CLICK_TOOLBELT_PROJECT_NAME,
                                         "{}.cfg".format(__namespace__))

        self.command.save_config(data)

        cfg = ConfigParser()
        cfg.read(expected_filename)
        self.assertTrue(cfg.has_section('login.ubuntu.com'))
        self.assertEqual(dict(cfg.items('login.ubuntu.com')), data)

    @patch('ssoclient.v2.http.requests.Session.request')
    def test_take_action_successful(self, mock_post):
        token_data = {
            'consumer_key': 'consumer-key',
            'consumer_secret': 'consumer-secret',
            'token_key': 'token-key',
            'token_secret': 'token-secret',
        }
        response = Response()
        response.status_code = 201
        response._content = json.dumps(token_data).encode('utf-8')
        mock_post.return_value = response

        data = {
            'email': 'foo@foo.com',
            'password': 'password',
            'token_name': 'consumer-key',
        }
        result = self.command.login(data)
        expected = {'success': True, 'body': token_data}
        self.assertEqual(result, expected)

    @patch('ssoclient.v2.http.requests.Session.request')
    def test_take_action_unsuccessful_api_exception(self, mock_post):
        error_data = {
            'message': 'Error during login.',
            'code': 'INVALID_CREDENTIALS',
            'extra': {},
        }
        response = Response()
        response.status_code = 401
        response.reason = 'UNAUTHORISED'
        response._content = json.dumps(error_data).encode('utf-8')
        mock_post.return_value = response

        data = {
            'email': 'foo@foo.com',
            'password': 'password',
            'token_name': 'consumer-key',
        }
        result = self.command.login(data)
        expected = {'success': False, 'body': error_data}
        self.assertEqual(result, expected)

    @patch('ssoclient.v2.http.requests.Session.request')
    def test_take_action_unsuccessful_unexpected_error(self, mock_post):
        error_data = {
            'message': 'Error during login.',
            'code': 'UNEXPECTED_ERROR_CODE',
            'extra': {},
        }
        response = Response()
        response.status_code = 401
        response.reason = 'UNAUTHORISED'
        response._content = json.dumps(error_data).encode('utf-8')
        mock_post.return_value = response

        data = {
            'email': 'foo@foo.com',
            'password': 'password',
            'token_name': 'consumer-key',
        }
        result = self.command.login(data)
        expected = {'success': False, 'body': error_data}
        self.assertEqual(result, expected)

    @patch('click_toolbelt.login.Login.save_config')
    @patch('click_toolbelt.login.Login.login')
    def test_take_action_success(self, mock_login, mock_save_config):
        parsed_args = namedtuple('parsed_args', 'email, password, otp')
        args = parsed_args(self.email, self.password, '123456')
        login_data = {
            'email': self.email,
            'password': self.password,
            'token_name': CLICK_TOOLBELT_PROJECT_NAME,
            'otp': '123456',
        }
        token_data = {
            'consumer_key': 'consumer-key',
            'consumer_secret': 'consumer-secret',
            'token_key': 'token-key',
            'token_secret': 'token-secret',
        }

        mock_login.return_value = {
            'success': True,
            'body': token_data,
        }

        self.command.take_action(args)

        mock_login.assert_called_once_with(login_data)
        mock_save_config.assert_called_once_with(token_data)
        # assert output
        self.mock_log.info.assert_called_once_with('Login successful.')

    @patch('click_toolbelt.login.Login.save_config')
    @patch('click_toolbelt.login.Login.login')
    def test_take_action_failure(self, mock_login, mock_save_config):
        login_data = {
            'email': self.email,
            'password': self.password,
            'token_name': CLICK_TOOLBELT_PROJECT_NAME,
        }
        error_data = {
            'code': 'UNAUTHORISED',
            'message': 'Provided email/password is not correct.',
            'extra': {},
        }

        mock_login.return_value = {
            'success': False,
            'body': error_data,
        }

        self.command.take_action(self.parsed_args)

        mock_login.assert_called_once_with(login_data)
        self.assertFalse(mock_save_config.called)
        # assert output
        self.mock_log.info.assert_called_once_with(
            'Login failed.\n'
            'Reason: %s\n'
            'Detail: %s',
            'UNAUTHORISED', 'Provided email/password is not correct.')



# Copyright 2014 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
from __future__ import absolute_import, unicode_literals
import json
import os
import tempfile
from collections import namedtuple

from mock import (
    ANY,
    DEFAULT,
    Mock,
    patch,
)
from requests import Response

from click_toolbelt import __namespace__
from click_toolbelt.constants import (
    UBUNTU_SSO_API_ROOT_URL,
)
from click_toolbelt.register import (
    Register,
)
from click_toolbelt.tests.test_common import BaseClickToolbeltCommandTestCase


class BaseRegisterCommandTestCase(BaseClickToolbeltCommandTestCase):
    command_class = Register

    def setUp(self):
        super(BaseRegisterCommandTestCase, self).setUp()

        # setup patches
        mock_environ = {
            'UBUNTU_SSO_API_ROOT_URL': UBUNTU_SSO_API_ROOT_URL,
        }
        patcher = patch('click_toolbelt.register.os.environ', mock_environ)
        patcher.start()
        self.addCleanup(patcher.stop)


class UploadFilesTestCase(BaseRegisterCommandTestCase):

    def setUp(self):
        super(UploadFilesTestCase, self).setUp()

        name = 'click_toolbelt.common.ClickToolbeltCommand.get_oauth_session'
        patcher = patch(name)
        mock_session = patcher.start()
        self.mock_post = mock_session.return_value.post
        self.addCleanup(patcher.stop)

        self.binary_file = self.get_temporary_file()

    def test_upload_files(self):
        mock_response = self.mock_post.return_value
        mock_response.ok = True
        mock_response.json.return_value = {
            'successful': True,
            'upload_id': 'some-valid-upload-id',
        }

        response = self.command.upload_files(self.binary_file.name)
        self.assertEqual(response, {
            'success': True,
            'errors': [],
            'upload_id': 'some-valid-upload-id',
            'binary_filesize': os.path.getsize(self.binary_file.name),
            'source_uploaded': False,
        })

    def test_upload_files_uses_environment_variables(self):
        with patch.dict(os.environ,
                        CLICK_UPDOWN_UPLOAD_URL='http://example.com'):
            upload_url = 'http://example.com/unscanned-upload/'
            self.command.upload_files(self.binary_file.name)
            self.mock_post.assert_called_once_with(
                upload_url, files={'binary': ANY})

    def test_upload_files_with_source_upload(self):
        mock_response = self.mock_post.return_value
        mock_response.ok = True
        mock_response.json.return_value = {
            'successful': True,
            'upload_id': 'some-valid-upload-id',
        }

        source_file = self.get_temporary_file()
        response = self.command.upload_files(self.binary_file.name,
                                             source_file.name)
        self.assertEqual(response, {
            'success': True,
            'errors': [],
            'upload_id': 'some-valid-upload-id',
            'binary_filesize': os.path.getsize(self.binary_file.name),
            'source_uploaded': True,
        })

    @patch('click_toolbelt.register.Register.get_oauth_session')
    def test_upload_files_with_invalid_oauth_session(self,
                                                     mock_get_oauth_session):
        mock_get_oauth_session.return_value = None
        response = self.command.upload_files(self.binary_file.name)
        self.assertEqual(response, {
            'success': False,
            'errors': ['No valid credentials found.'],
        })
        self.assertFalse(self.mock_post.called)

    def test_upload_files_error_response(self):
        mock_response = self.mock_post.return_value
        mock_response.ok = False
        mock_response.reason = '500 INTERNAL SERVER ERROR'
        mock_response.text = 'server failed'

        response = self.command.upload_files(self.binary_file.name)
        self.assertEqual(response, {
            'success': False,
            'errors': ['server failed'],
        })

    def test_upload_files_handle_malformed_response(self):
        mock_response = self.mock_post.return_value
        mock_response.json.return_value = {'successful': False}

        response = self.command.upload_files(self.binary_file.name)
        err = KeyError('upload_id')
        self.assertEqual(response, {
            'success': False,
            'errors': [str(err)],
        })


class RegisterAppTestCase(BaseRegisterCommandTestCase):

    def setUp(self):
        super(RegisterAppTestCase, self).setUp()

        name = 'click_toolbelt.common.ClickToolbeltCommand.get_oauth_session'
        patcher = patch(name)
        mock_session = patcher.start()
        self.mock_get = mock_session.return_value.get
        self.mock_post = mock_session.return_value.post
        self.addCleanup(patcher.stop)

        self.data = {
            'upload_id': 'some-valid-upload-id',
            'binary_filesize': 123456,
            'source_uploaded': False,
        }

        patcher = patch.multiple(
            'click_toolbelt.register',
            SCAN_STATUS_POLL_DELAY=0.0001)
        patcher.start()
        self.addCleanup(patcher.stop)

    @patch('click_toolbelt.register.Register.get_oauth_session')
    def test_register_app_with_invalid_oauth_session(self,
                                                     mock_get_oauth_session):
        mock_get_oauth_session.return_value = None
        response = self.command.register_app(self.data)
        self.assertEqual(response, {
            'success': False,
            'errors': ['No valid credentials found.'],
            'application_url': '',
        })

    def test_register_app_uses_environment_variables(self):
        with patch.dict(os.environ,
                        MYAPPS_API_ROOT_URL='http://example.com'):
            register_url = 'http://example.com/click-app/'
            data = {
                'updown_id': self.data['upload_id'],
                'binary_filesize': self.data['binary_filesize'],
                'source_uploaded': self.data['source_uploaded'],
            }
            self.command.register_app(self.data)
            self.mock_post.assert_called_once_with(
                register_url, data=data, files=[])

    def test_register_app(self):
        mock_response = self.mock_post.return_value
        mock_response.ok = True
        mock_response.json.return_value = {
            'success': True,
            'status_url': 'http://example.com/status/'
        }

        mock_status_response = self.mock_get.return_value
        mock_status_response.ok = True
        mock_status_response.json.return_value = {
            'completed': True
        }

        response = self.command.register_app(self.data)
        self.assertEqual(response, {
            'success': True,
            'errors': [],
            'application_url': '',
        })

    def test_register_app_error_response(self):
        mock_response = self.mock_post.return_value
        mock_response.ok = False
        mock_response.reason = '500 INTERNAL SERVER ERROR'
        mock_response.text = 'server failure'

        response = self.command.register_app(self.data)
        self.assertEqual(response, {
            'success': False,
            'errors': ['server failure'],
            'application_url': '',
        })

    def test_register_app_handle_malformed_response(self):
        mock_response = self.mock_post.return_value
        mock_response.ok = True
        mock_response.json.return_value = {}

        response = self.command.register_app(self.data)
        err = KeyError('status_url')
        self.assertEqual(response, {
            'success': False,
            'errors': [str(err)],
            'application_url': '',
        })

    def test_register_app_with_errors_during_scan(self):
        mock_response = self.mock_post.return_value
        mock_response.ok = True
        mock_response.json.return_value = {
            'success': True,
            'status_url': 'http://example.com/status/'
        }

        mock_status_response = self.mock_get.return_value
        mock_status_response.ok = True
        mock_status_response.json.return_value = {
            'completed': True,
            'message': 'some error',
            'application_url': 'http://example.com/myapp',
        }

        response = self.command.register_app(self.data)
        self.assertEqual(response, {
            'success': False,
            'errors': ['some error'],
            'application_url': 'http://example.com/myapp'
        })

    def test_register_app_poll_status(self):
        mock_response = self.mock_post.return_value
        mock_response.ok = True
        mock_response.return_value = {
            'success': True,
            'status_url': 'http://example.com/status/'
        }

        response_not_completed = Response()
        response_not_completed.status_code = 200
        response_not_completed.encoding = 'utf-8'
        response_not_completed._content = json.dumps(
            {'completed': False, 'application_url': ''}).encode('utf-8')
        response_completed = Response()
        response_completed.status_code = 200
        response_completed.encoding = 'utf-8'
        response_completed._content = json.dumps(
            {'completed': True,
             'application_url': 'http://example.org'}).encode('utf-8')
        self.mock_get.side_effect = [
            response_not_completed,
            response_not_completed,
            response_completed,
        ]
        response = self.command.register_app(self.data)
        self.assertEqual(response, {
            'success': True,
            'errors': [],
            'application_url': 'http://example.org',
        })
        self.assertEqual(self.mock_get.call_count, 3)

    def test_register_app_ignore_non_ok_responses(self):
        mock_response = self.mock_post.return_value
        mock_response.ok = True
        mock_response.return_value = {
            'success': True,
            'status_url': 'http://example.com/status/',
        }

        ok_response = Response()
        ok_response.status_code = 200
        ok_response.encoding = 'utf-8'
        ok_response._content = json.dumps(
            {'completed': True}).encode('utf-8')
        nok_response = Response()
        nok_response.status_code = 503

        self.mock_get.side_effect = [nok_response, nok_response, ok_response]
        response = self.command.register_app(self.data)
        self.assertEqual(response, {
            'success': True,
            'errors': [],
            'application_url': '',
        })
        self.assertEqual(self.mock_get.call_count, 3)

    def test_register_app_abort_polling(self):
        mock_response = self.mock_post.return_value
        mock_response.ok = True
        mock_response.json.return_value = {
            'success': True,
            'status_url': 'http://example.com/status/',
        }

        mock_status_response = self.mock_get.return_value
        mock_status_response.ok = True
        mock_status_response.json.return_value = {
            'completed': False
        }
        response = self.command.register_app(self.data)
        self.assertEqual(response, {
            'success': False,
            'errors': [
                'Package scan took too long.',
                'Please check the status later at: '
                'http://example.com/status/.',
            ],
            'application_url': '',
        })

    def test_register_app_with_metadata(self):
        self.command.register_app(self.data,
                                  metadata={'changelog': 'some changes',
                                            'tagline': 'a tagline'})
        self.mock_post.assert_called_once_with(
            ANY,
            data={
                'updown_id': self.data['upload_id'],
                'binary_filesize': self.data['binary_filesize'],
                'source_uploaded': self.data['source_uploaded'],
                'changelog': 'some changes',
                'tagline': 'a tagline',
            },
            files=[],
        )

    def test_register_app_ignore_special_attributes_in_metadata(self):
        self.command.register_app(
            self.data, metadata={
                'changelog': 'some changes',
                'tagline': 'a tagline',
                'upload_id': 'my-own-id',
                'binary_filesize': 0,
                'source_uploaded': True,
            })
        self.mock_post.assert_called_once_with(
            ANY,
            data={
                'updown_id': self.data['upload_id'],
                'binary_filesize': self.data['binary_filesize'],
                'source_uploaded': self.data['source_uploaded'],
                'changelog': 'some changes',
                'tagline': 'a tagline',
            },
            files=[],
        )

    @patch('click_toolbelt.register.open')
    def test_register_app_with_icon(self, mock_open):
        with tempfile.NamedTemporaryFile() as icon:
            mock_open.return_value = icon

            self.command.register_app(
                self.data, metadata={
                    'icon_256': icon.name,
                }
            )
            self.mock_post.assert_called_once_with(
                ANY,
                data={
                    'updown_id': self.data['upload_id'],
                    'binary_filesize': self.data['binary_filesize'],
                    'source_uploaded': self.data['source_uploaded'],
                },
                files=[
                    ('icon_256', icon),
                ],
            )

    @patch('click_toolbelt.register.open')
    def test_register_app_with_screenshots(self, mock_open):
        screenshot1 = tempfile.NamedTemporaryFile()
        screenshot2 = tempfile.NamedTemporaryFile()
        mock_open.side_effect = [screenshot1, screenshot2]

        self.command.register_app(
            self.data, metadata={
                'screenshots': [screenshot1.name, screenshot2.name],
            }
        )
        self.mock_post.assert_called_once_with(
            ANY,
            data={
                'updown_id': self.data['upload_id'],
                'binary_filesize': self.data['binary_filesize'],
                'source_uploaded': self.data['source_uploaded'],
            },
            files=[
                ('screenshots', screenshot1),
                ('screenshots', screenshot2),
            ],
        )


class RegisterCommandTestCase(BaseRegisterCommandTestCase):

    def setUp(self):
        super(RegisterCommandTestCase, self).setUp()

        patcher = patch('click_toolbelt.register.Register.log')
        self.mock_log = patcher.start()
        self.addCleanup(patcher.stop)

        self.parsed_args = namedtuple(
            'parsed_args',
            'binary_filename, source_filename, metadata_filename')
        self.args = self.parsed_args('binary', None, None)

    def test_parser(self):
        parser = self.command.get_parser(__namespace__)
        # binary_filename is required
        self.assertEqual(parser._actions[1].dest, 'binary_filename')
        self.assertTrue(parser._actions[1].required)
        # source_filename is optional
        self.assertEqual(parser._actions[2].dest, 'source_filename')
        # this is a positional argument
        self.assertEqual(parser._actions[2].option_strings, [])
        self.assertFalse(parser._actions[2].required)
        self.assertEqual(parser._actions[3].dest, 'source_filename')
        # this is a named argument
        self.assertEqual(parser._actions[3].option_strings, ['--source'])
        self.assertFalse(parser._actions[3].required)
        # metadata_filename is optional
        self.assertEqual(parser._actions[4].dest, 'metadata_filename')
        # this is a positional argument
        self.assertEqual(parser._actions[4].option_strings, [])
        self.assertFalse(parser._actions[4].required)
        self.assertEqual(parser._actions[5].dest, 'metadata_filename')
        # this is a named argument
        self.assertEqual(parser._actions[5].option_strings, ['--metadata'])
        self.assertFalse(parser._actions[5].required)

    def test_take_action(self):
        binary_filename = self.args.binary_filename
        source_filename = self.args.source_filename

        with patch.multiple(self.command, upload_files=DEFAULT,
                            register_app=DEFAULT):

            self.command.take_action(self.args)

            self.command.upload_files.assert_called_once_with(
                binary_filename, source_filename=source_filename)
            self.command.register_app.assert_called_once_with(
                self.command.upload_files.return_value,
                metadata={})
            self.mock_log.info.assert_any_call(
                'Application registered successfully.'
            )

    def test_take_action_with_metadata_file(self):
        mock_metadata_file = tempfile.NamedTemporaryFile()
        with open(mock_metadata_file.name, 'wb') as mock_file:
            data = json.dumps({'changelog': 'some changes'})
            mock_file.write(data.encode('utf-8'))
            mock_file.flush()

        with patch.multiple(self.command, upload_files=DEFAULT,
                            register_app=DEFAULT):

            args = self.parsed_args('binary', 'source',
                                    mock_metadata_file.name)
            self.command.take_action(args)

            self.command.upload_files.assert_called_once_with(
                'binary', source_filename='source')
            self.command.register_app.assert_called_once_with(
                self.command.upload_files.return_value,
                metadata={'changelog': 'some changes'})
            self.mock_log.info.assert_any_call(
                'Application registered successfully.'
            )

    def test_take_action_with_metadata_file_without_source_file(self):
        mock_metadata_file = tempfile.NamedTemporaryFile()
        with open(mock_metadata_file.name, 'wb') as mock_file:
            data = json.dumps({'changelog': 'some changes'})
            mock_file.write(data.encode('utf-8'))
            mock_file.flush()

        with patch.multiple(self.command, upload_files=DEFAULT,
                            register_app=DEFAULT):

            args = self.parsed_args('binary', None,
                                    mock_metadata_file.name)
            self.command.take_action(args)

            self.command.upload_files.assert_called_once_with(
                'binary', source_filename=None)
            self.command.register_app.assert_called_once_with(
                self.command.upload_files.return_value,
                metadata={'changelog': 'some changes'})
            self.mock_log.info.assert_any_call(
                'Application registered successfully.'
            )

    def test_take_action_with_error_during_upload(self):
        binary_filename = self.args.binary_filename
        source_filename = self.args.source_filename

        with patch.multiple(self.command, upload_files=DEFAULT,
                            register_app=DEFAULT):

            self.command.upload_files.return_value = {
                'success': False,
                'errors': ['some error']
            }

            self.command.take_action(self.args)

            self.command.upload_files.assert_called_once_with(
                binary_filename, source_filename=source_filename)
            self.assertFalse(self.command.register_app.called)
            self.mock_log.info.assert_any_call(
                'Upload failed:\n\n%s\n', 'some error')

    def test_take_action_with_error_during_registration(self):
        binary_filename = self.args.binary_filename
        source_filename = self.args.source_filename

        with patch.multiple(self.command, upload_files=DEFAULT,
                            register_app=DEFAULT):

            self.command.register_app.return_value = {
                'success': False,
                'errors': ['some error'],
            }

            self.command.take_action(self.args)

            self.command.upload_files.assert_called_once_with(
                binary_filename, source_filename=source_filename)
            self.command.register_app.assert_called_once_with(
                self.command.upload_files.return_value,
                metadata={})
            self.mock_log.info.assert_any_call(
                'Registration did not complete.')
            self.mock_log.info.assert_any_call(
                'Some errors were detected:\n\n%s\n\n',
                'some error')

    def test_take_action_shows_application_url(self):
        with patch.multiple(self.command, upload_files=DEFAULT,
                            register_app=DEFAULT):

            self.command.register_app.return_value = {
                'success': True,
                'errors': [],
                'application_url': 'http://example.com/',
            }

            self.command.take_action(self.args)

            self.mock_log.info.assert_any_call(
                'Please check out the application at: %s.\n',
                'http://example.com/')



# Copyright 2014 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
from __future__ import absolute_import, unicode_literals
import os
import tempfile
from unittest import TestCase

from mock import Mock, call, patch
from requests_oauthlib import OAuth1Session

from click_toolbelt.common import ClickToolbeltCommand, retry
from click_toolbelt.compat import ConfigParser


class BaseClickToolbeltCommandTestCase(TestCase):
    command_class = ClickToolbeltCommand

    def setUp(self):
        super(BaseClickToolbeltCommandTestCase, self).setUp()
        app = None
        args = None
        self.command = self.command_class(app, args)

    def get_temporary_file(self):
        return tempfile.NamedTemporaryFile()


class ClickToolbeltCommandTestCase(BaseClickToolbeltCommandTestCase):

    def test_take_action(self):
        self.assertEqual(self.command.take_action(None), None)


class GetConfigTestCase(BaseClickToolbeltCommandTestCase):

    def setUp(self):
        super(GetConfigTestCase, self).setUp()
        cfg_file = self.get_temporary_file()
        self.filename = cfg_file.name
        patcher = patch('click_toolbelt.common.os.path.join')
        mock_join = patcher.start()
        mock_join.return_value = self.filename
        self.addCleanup(patcher.stop)

        # make sure env is not overwritten
        patcher = patch.object(os, 'environ', {})
        patcher.start()
        self.addCleanup(patcher.stop)

    def test_get_config_with_no_existing_file(self):
        data = self.command.get_config()
        self.assertEqual(data, {})

    def test_get_config_with_no_existing_section(self):
        cfg = ConfigParser()
        cfg.add_section('some.domain')
        cfg.set('some.domain', 'foo', '1')
        cfg.write(open(self.filename, 'w'))

        data = self.command.get_config()
        self.assertEqual(data, {})

    def test_get_config(self):
        cfg = ConfigParser()
        cfg.add_section('login.ubuntu.com')
        cfg.set('login.ubuntu.com', 'foo', '1')
        cfg.write(open(self.filename, 'w'))

        data = self.command.get_config()
        self.assertEqual(data, {'foo': '1'})


class GetOAuthSessionTestCase(BaseClickToolbeltCommandTestCase):

    def setUp(self):
        super(GetOAuthSessionTestCase, self).setUp()
        patcher = patch(
            'click_toolbelt.common.ClickToolbeltCommand.get_config')
        self.mock_get_config = patcher.start()
        self.addCleanup(patcher.stop)

    def test_get_oauth_session_when_no_config(self):
        self.mock_get_config.return_value = {}
        session = self.command.get_oauth_session()
        self.assertIsNone(session)

    def test_get_oauth_session_when_partial_config(self):
        self.mock_get_config.return_value = {
            'consumer_key': 'consumer-key',
            'consumer_secret': 'consumer-secret',
        }
        session = self.command.get_oauth_session()
        self.assertIsNone(session)

    def test_get_oauth_session(self):
        self.mock_get_config.return_value = {
            'consumer_key': 'consumer-key',
            'consumer_secret': 'consumer-secret',
            'token_key': 'token-key',
            'token_secret': 'token-secret',
        }
        session = self.command.get_oauth_session()
        self.assertIsInstance(session, OAuth1Session)
        self.assertEqual(session.auth.client.client_key, 'consumer-key')
        self.assertEqual(session.auth.client.client_secret, 'consumer-secret')
        self.assertEqual(session.auth.client.resource_owner_key, 'token-key')
        self.assertEqual(session.auth.client.resource_owner_secret,
                         'token-secret')


class RetryDecoratorTestCase(TestCase):

    def target(self, *args, **kwargs):
        return dict(args=args, kwargs=kwargs)

    def test_retry(self):
        result, aborted = retry()(self.target)()
        self.assertEqual(result, dict(args=(), kwargs={}))
        self.assertEqual(aborted, False)

    @patch('click_toolbelt.common.time.sleep')
    def test_retry_small_backoff(self, mock_sleep):
        mock_terminator = Mock()
        mock_terminator.return_value = False

        delay = 0.001
        result, aborted = retry(mock_terminator, retries=2,
                                delay=delay)(self.target)()

        self.assertEqual(result, dict(args=(), kwargs={}))
        self.assertEqual(aborted, True)
        self.assertEqual(mock_terminator.call_count, 3)
        self.assertEqual(mock_sleep.mock_calls, [
            call(delay),
            call(delay * 2),
        ])

    def test_retry_abort(self):
        mock_terminator = Mock()
        mock_terminator.return_value = False
        mock_logger = Mock()

        result, aborted = retry(mock_terminator, delay=0.001, backoff=1,
                                logger=mock_logger)(self.target)()

        self.assertEqual(result, dict(args=(), kwargs={}))
        self.assertEqual(aborted, True)
        self.assertEqual(mock_terminator.call_count, 4)
        self.assertEqual(mock_logger.warning.call_count, 3)

    def test_retry_with_invalid_retries(self):
        for value in (0.1, -1):
            with self.assertRaises(ValueError) as ctx:
                retry(retries=value)(self.target)
            self.assertEqual(
                str(ctx.exception),
                'retries value must be a positive integer or zero')

    def test_retry_with_negative_delay(self):
        with self.assertRaises(ValueError) as ctx:
            retry(delay=-1)(self.target)
        self.assertEqual(str(ctx.exception),
                         'delay value must be positive')

    def test_retry_with_invalid_backoff(self):
        for value in (-1, 0, 0.1):
            with self.assertRaises(ValueError) as ctx:
                retry(backoff=value)(self.target)
            self.assertEqual(str(ctx.exception),
                             'backoff value must be a positive integer')



# -*- coding: utf-8 -*-
# Copyright 2014 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
from __future__ import absolute_import, unicode_literals
from collections import namedtuple
from unittest import TestCase

from mock import call, patch

from click_toolbelt.info import (
    Info,
)


class InfoCommandTestCase(TestCase):

    def setUp(self):
        super(InfoCommandTestCase, self).setUp()
        app = None
        args = None
        self.command = Info(app, args)

        patcher = patch('click_toolbelt.info.Info.log')
        self.mock_log = patcher.start()
        self.addCleanup(patcher.stop)

        patcher = patch('click_toolbelt.info.requests.get')
        self.mock_get = patcher.start()
        self.mock_response = self.mock_get.return_value
        self.addCleanup(patcher.stop)

        self.parsed_args = namedtuple('parsed_args', 'topic')
        self.args = self.parsed_args(None)

    def test_parser(self):
        parser = self.command.get_parser('prog_name')
        # only one argument -- the first item is the default help option
        self.assertEqual(len(parser._actions), 2)
        self.assertEqual(parser._actions[0].dest, 'help')
        # topic is optional
        self.assertEqual(parser._actions[1].dest, 'topic')
        self.assertFalse(parser._actions[1].required)

    def test_get_info(self):
        expected = {
            'success': True,
            'errors': [],
            'data': {'version': 1},
        }
        self.mock_response.ok = True
        self.mock_response.json.return_value = {'version': 1}
        data = self.command.get_info()
        self.assertEqual(data, expected)

    def test_get_info_with_error_response(self):
        expected = {
            'success': False,
            'errors': ['some error'],
            'data': None,
        }
        self.mock_response.ok = False
        self.mock_response.text = 'some error'
        data = self.command.get_info()
        self.assertEqual(data, expected)

    def test_get_info_uses_environment_variables(self):
        with patch('click_toolbelt.info.os.environ',
                   {'MYAPPS_API_ROOT_URL': 'http://example.com'}):
            self.command.get_info()
        self.mock_get.assert_called_once_with('http://example.com')

    def test_take_action(self):
        with patch.object(self.command, 'get_info') as mock_get_info:
            mock_get_info.return_value = {
                'success': True, 'errors': [], 'data': {'version': 1},
            }
            self.command.take_action(self.args)
        self.assertEqual(self.mock_log.info.call_args_list, [
            call('API info:'),
            call('  %s: %s', 'version', 1),
        ])

    def _test_take_action_with_topic(self, data, topic, expected):
        with patch.object(self.command, 'get_info') as mock_get_info:
            mock_get_info.return_value = {
                'success': True, 'errors': [], 'data': data,
            }
            args = self.parsed_args(topic)
            self.command.take_action(args)
        self.mock_log.info.assert_any_call('API info:')
        self.mock_log.info.assert_any_call(
            '  %s: %s', topic, expected)

    def test_take_action_with_version(self):
        data = {'version': 1}
        self._test_take_action_with_topic(data, 'version', 1)

    def test_take_action_with_department(self):
        expected = ['foo']
        data = {'/dev/api/click-app/': {'department': expected}}
        self._test_take_action_with_topic(data, 'department', expected)

    def test_take_action_with_license(self):
        expected = ['foo']
        data = {'/dev/api/click-app/': {'license': expected}}
        self._test_take_action_with_topic(data, 'license', expected)

    def test_take_action_with_country(self):
        expected = [['FO', 'foo']]
        data = {'/dev/api/click-app/': {'country': expected}}
        self._test_take_action_with_topic(data, 'country', expected)

    def test_take_action_with_error(self):
        with patch.object(self.command, 'get_info') as mock_get_info:
            mock_get_info.return_value = {
                'success': False, 'errors': ['some error'], 'data': None,
            }
            self.command.take_action(self.args)
        self.mock_log.info.assert_called_once_with(
            'Could not get information. An error ocurred:\n\n%s\n\n',
            'some error')



# -*- coding: utf-8 -*-
# Copyright 2013 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
from __future__ import absolute_import, unicode_literals
import logging
import os

from cliff.command import Command
from ssoclient.v2 import (
    ApiException,
    UnexpectedApiError,
    V2ApiClient,
)
from xdg.BaseDirectory import save_config_path

from click_toolbelt import __namespace__
from click_toolbelt.constants import (
    CLICK_TOOLBELT_PROJECT_NAME,
    UBUNTU_SSO_API_ROOT_URL,
)
from click_toolbelt.compat import ConfigParser, urlparse


class Login(Command):
    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super(Login, self).get_parser(prog_name)
        parser.add_argument('email')
        parser.add_argument('password')
        parser.add_argument('otp', nargs='?')
        return parser

    def save_config(self, data):
        config_dir = save_config_path(CLICK_TOOLBELT_PROJECT_NAME)

        filename = os.path.join(config_dir, "%s.cfg" % __namespace__)
        parser = ConfigParser()
        if os.path.exists(filename):
            parser.read(filename)

        api_endpoint = os.environ.get(
            'UBUNTU_SSO_API_ROOT_URL', UBUNTU_SSO_API_ROOT_URL)
        location = urlparse(api_endpoint).netloc
        if not parser.has_section(location):
            parser.add_section(location)

        for key, value in data.items():
            parser.set(location, key, value)

        parser.write(open(filename, 'w'))

    def login(self, data):
        result = {
            'success': False,
            'body': None,
        }

        api_endpoint = os.environ.get(
            'UBUNTU_SSO_API_ROOT_URL', UBUNTU_SSO_API_ROOT_URL)
        client = V2ApiClient(endpoint=api_endpoint)
        try:
            response = client.login(data=data)
            result['body'] = response
            result['success'] = True
        except ApiException as err:
            result['body'] = err.body
        except UnexpectedApiError as err:
            result['body'] = err.json_body
        return result

    def take_action(self, parsed_args):
        login_data = {
            'email': parsed_args.email,
            'password': parsed_args.password,
            'token_name': CLICK_TOOLBELT_PROJECT_NAME,
        }
        if parsed_args.otp:
            login_data['otp'] = parsed_args.otp

        response = self.login(login_data)
        if response.get('success', False):
            self.log.info('Login successful.')
            self.save_config(response['body'])
        else:
            error = response['body']
            self.log.info(
                'Login failed.\n'
                'Reason: %s\n'
                'Detail: %s',
                error['code'], error['message'])



# Copyright 2013 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
from __future__ import absolute_import, unicode_literals
import sys
from unittest import TestCase

from mock import patch

from click_toolbelt import __version__
from click_toolbelt.toolbelt import ClickToolbelt, main


class ClickToolbeltTestCase(TestCase):
    def test_constructor(self):
        toolbelt = ClickToolbelt()
        self.assertEqual(toolbelt.parser.description,
                         'Tools for working with click packages')
        self.assertEqual(toolbelt.command_manager.namespace, 'click_toolbelt')
        with self.assertRaises(SystemExit):
            self.assertIn(__version__, toolbelt.run(['--version']))


class MainTestCase(TestCase):
    def test_with_args(self):
        with patch('click_toolbelt.toolbelt.ClickToolbelt.run') as mock_run:
            args = []
            main(args)
            mock_run.assert_called_with(args)

    def test_no_args_uses_sys(self):
        with patch('click_toolbelt.toolbelt.ClickToolbelt.run') as mock_run:
            main()
            mock_run.assert_called_with(sys.argv[1:])



# Copyright 2013 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
from __future__ import absolute_import, unicode_literals

__namespace__ = 'click_toolbelt'
__version__ = '0.2'



# Copyright 2013 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
#!/usr/bin/env python
from setuptools import setup, find_packages

from click_toolbelt import __namespace__, __version__

try:
    long_description = open('README.txt', 'rt').read()
except IOError:
    long_description = ''


setup(
    name='click-toolbelt',
    version=__version__,

    description='Click App Toolbelt',
    long_description=long_description,

    author='Ricardo Kirkner',
    author_email='ricardo.kirkner@canonical.com',

    url='https://launchpad.net/click-toolbelt',
    download_url='https://launchpad.net/click-toolbelt',

    classifiers=['Development Status :: 4 - Beta',
                 'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
                 'Programming Language :: Python',
                 'Programming Language :: Python :: 2',
                 'Programming Language :: Python :: 2.7',
                 'Programming Language :: Python :: 3',
                 'Programming Language :: Python :: 3.3',
                 'Intended Audience :: Developers',
                 'Environment :: Console',
                 ],

    platforms=['Any'],

    scripts=[],

    provides=[],
    install_requires=[
        'cliff',
        'pyxdg',
        'requests-oauthlib',
        'requests',
        'ssoclient',
    ],

    namespace_packages=[],
    packages=find_packages(),
    include_package_data=True,

    entry_points={
        'console_scripts': [
            'click-toolbelt = click_toolbelt.toolbelt:main'
        ],
        __namespace__: [
            'login = click_toolbelt.login:Login',
            'upload = click_toolbelt.upload:Upload',
            'register = click_toolbelt.register:Register',
            'info = click_toolbelt.info:Info',
        ],
    },

    zip_safe=False,

    test_suite='click_toolbelt.tests',
    tests_require=[
        'mock',
    ],
)



# Copyright 2013 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
from __future__ import absolute_import, unicode_literals
import json
from collections import namedtuple

from mock import patch
from requests import Response

from click_toolbelt import __namespace__
from click_toolbelt.constants import (
    UBUNTU_SSO_API_ROOT_URL,
)
from click_toolbelt.upload import (
    CLICK_UPDOWN_UPLOAD_URL,
    MYAPPS_API_ROOT_URL,
    Upload,
)
from click_toolbelt.tests.test_common import BaseClickToolbeltCommandTestCase


class BaseUploadCommandTestCase(BaseClickToolbeltCommandTestCase):
    command_class = Upload

    def setUp(self):
        super(BaseUploadCommandTestCase, self).setUp()
        self.app_name = 'namespace.package_name'
        self.app_archs = ['all']
        self.app_framework = 'ubuntu-sdk-13.10'
        self.app_version = '1'
        self.app_changelog = 'Initial version.'
        self.uploaded_callback_url = MYAPPS_API_ROOT_URL + 'click-uploaded/1/'

        # setup patches
        mock_environ = {
            'UBUNTU_SSO_API_ROOT_URL': UBUNTU_SSO_API_ROOT_URL,
        }
        patcher = patch('click_toolbelt.upload.os.environ', mock_environ)
        patcher.start()
        self.addCleanup(patcher.stop)


class GetUploadDataTestCase(BaseUploadCommandTestCase):

    def setUp(self):
        super(GetUploadDataTestCase, self).setUp()

        patcher = patch('click_toolbelt.upload.Upload.get_config')
        patcher.start()
        self.addCleanup(patcher.stop)

        patcher = patch('click_toolbelt.upload.Upload.get_oauth_session')
        mock_session = patcher.start()
        self.mock_post = mock_session.return_value.post
        self.addCleanup(patcher.stop)

        mock_environ = {'MYAPPS_API_ROOT_URL': MYAPPS_API_ROOT_URL}
        patcher = patch('click_toolbelt.upload.os.environ', mock_environ)
        patcher.start()
        self.addCleanup(patcher.stop)

    def test_get_upload_data(self):
        expected = {
            'success': True,
            'errors': [],
            'binary_path': '/path/to/binary',
            'source_path': '/path/to/source',
            'signature': 'signature',
            'application_id': 1,
        }

        response = Response()
        response.status_code = 200
        response._content = json.dumps(expected).encode('utf-8')
        self.mock_post.return_value = response

        data = self.command.get_upload_data(self.app_name,
                                            self.app_version,
                                            self.app_changelog,
                                            self.app_archs,
                                            self.app_framework)
        self.assertEqual(data, expected)

    def test_get_upload_data_for_unknown_app(self):
        response = Response()
        response.status_code = 404
        response._content = (
            'The requested Click Package could not be found.'.encode('utf-8'))
        self.mock_post.return_value = response

        data = self.command.get_upload_data(self.app_name,
                                            self.app_version,
                                            self.app_changelog,
                                            self.app_archs,
                                            self.app_framework)
        self.assertEqual(data, {
            'success': False,
            'errors': ['The requested Click Package could not be found.']})

    def test_get_upload_data_resilient(self):
        self.mock_post.side_effect = Exception('some http error')

        data = self.command.get_upload_data(self.app_name,
                                            self.app_version,
                                            self.app_changelog,
                                            self.app_archs,
                                            self.app_framework)
        self.assertEqual(data, {'success': False,
                                'errors': ['some http error']})

    def test_get_upload_data_uses_default_myapps_url(self):
        upload_url = MYAPPS_API_ROOT_URL + "click-upload/%s/" % (
            self.app_name,)
        self.command.get_upload_data(
            self.app_name, self.app_version, self.app_changelog,
            self.app_archs, self.app_framework)
        self.mock_post.assert_called_once_with(
            upload_url, data={'version': self.app_version,
                              'architectures': self.app_archs,
                              'changelog': self.app_changelog,
                              'framework': self.app_framework})

    def test_get_upload_data_uses_environment_variable(self):
        upload_url = MYAPPS_API_ROOT_URL + "click-upload/%s/" % (
            self.app_name,)
        self.command.get_upload_data(
            self.app_name, self.app_version, self.app_changelog,
            self.app_archs, self.app_framework)
        self.mock_post.assert_called_once_with(
            upload_url, data={'version': self.app_version,
                              'architectures': self.app_archs,
                              'changelog': self.app_changelog,
                              'framework': self.app_framework})

    @patch('click_toolbelt.upload.Upload.get_oauth_session')
    def test_get_upload_data_with_invalid_session(self,
                                                  mock_get_oauth_session):
        mock_get_oauth_session.return_value = None
        result = self.command.get_upload_data(
            self.app_name, self.app_version, self.app_changelog,
            self.app_archs, self.app_framework)
        self.assertEqual(result, {'success': False,
                                  'errors': ['No valid credentials found.']})
        self.assertFalse(self.mock_post.called)


class MarkUploadedTestCase(BaseUploadCommandTestCase):

    def setUp(self):
        super(MarkUploadedTestCase, self).setUp()

        patcher = patch('click_toolbelt.upload.Upload.get_oauth_session')
        mock_session = patcher.start()
        self.mock_post = mock_session.return_value.post
        self.addCleanup(patcher.stop)

        mock_environ = {'MYAPPS_API_ROOT_URL': MYAPPS_API_ROOT_URL}
        patcher = patch('click_toolbelt.upload.os.environ', mock_environ)
        patcher.start()
        self.addCleanup(patcher.stop)

    def test_mark_uploaded(self):
        response_data = {'success': True, 'errors': []}
        response = Response()
        response.status_code = 200
        response.reason = 'OK'
        response._content = json.dumps(response_data).encode('utf-8')
        self.mock_post.return_value = response

        result = self.command.mark_uploaded(
            self.uploaded_callback_url, self.app_name, self.app_version, 42)
        self.assertEqual(result, response_data)
        self.mock_post.assert_called_once_with(
            self.uploaded_callback_url, data={'version': self.app_version,
                                              'binary': 42})

    @patch('click_toolbelt.upload.Upload.get_oauth_session')
    def test_mark_uploaded_with_invalid_session(self, mock_get_oauth_session):
        mock_get_oauth_session.return_value = None
        result = self.command.mark_uploaded(
            self.uploaded_callback_url, self.app_name, self.app_version, 42)
        self.assertEqual(result, {'success': False,
                                  'errors': ['No valid credentials found.']})
        self.assertFalse(self.mock_post.called)

    def test_mark_uploaded_unsuccessful(self):
        response_data = {'foo': 'bar'}
        response = Response()
        response.status_code = 500
        response.reason = 'INTERNAL SERVER ERROR'
        response._content = json.dumps(response_data).encode('utf-8')
        self.mock_post.return_value = response

        result = self.command.mark_uploaded(
            self.uploaded_callback_url, self.app_name, self.app_version, 42)
        self.assertEqual(result, {'success': False,
                                  'errors': [json.dumps(response_data)]})
        self.mock_post.assert_called_once_with(
            self.uploaded_callback_url, data={'version': self.app_version,
                                              'binary': 42})

    @patch('click_toolbelt.upload.Upload.log')
    def test_mark_uploaded_error_during_post(self, mock_log):
        self.mock_post.side_effect = Exception('the error')

        result = self.command.mark_uploaded(
            self.uploaded_callback_url, self.app_name, self.app_version, 42)
        self.assertEqual(result, {'success': False, 'errors': ['the error']})
        self.mock_post.assert_called_once_with(
            self.uploaded_callback_url, data={'version': self.app_version,
                                              'binary': 42})
        mock_log.exception.assert_called_once_with(
            "There was an error marking application '%s' version '%s' "
            "as uploaded.", self.app_name, self.app_version)


class UploadCommandTestCase(BaseUploadCommandTestCase):

    def setUp(self):
        super(UploadCommandTestCase, self).setUp()

        patcher = patch('click_toolbelt.upload.Upload.log')
        self.mock_log = patcher.start()
        self.addCleanup(patcher.stop)

        patcher = patch('click_toolbelt.upload.requests.post')
        self.mock_post = patcher.start()
        self.addCleanup(patcher.stop)

        patcher = patch('click_toolbelt.upload.Upload.get_upload_data')
        self.mock_get_upload_data = patcher.start()
        self.mock_get_upload_data.return_value = {
            'success': True,
            'errors': [],
            'binary_path': '/path/to/binary',
            'source_path': '/path/to/source',
            'signature': 'signature',
            'application_id': 1,
        }
        self.addCleanup(patcher.stop)

        patcher = patch('click_toolbelt.upload.open')
        self.mock_open = patcher.start()
        self.addCleanup(patcher.stop)

        patcher = patch('click_toolbelt.upload.os.path.getsize')
        patcher.start()
        self.addCleanup(patcher.stop)

        parsed_args = namedtuple(
            'parsed_args',
            'application_name, version, filename, changelog, arch, framework')

        filename = 'package.click'
        self.args = parsed_args(
            self.app_name, self.app_version, filename, self.app_changelog,
            self.app_archs, self.app_framework)

    def test_parser(self):
        parser = self.command.get_parser(__namespace__)
        cmd_args = ['application_name', 'version', 'filename', 'changelog']
        for i, name in enumerate(cmd_args):
            # argument 0 is builtin --help
            # start comparing from first extra argument
            self.assertEqual(parser._actions[i + 1].dest, name)
            self.assertTrue(parser._actions[i + 1].required)
        # framework is optional
        self.assertEqual(parser._actions[-2].dest, 'framework')
        self.assertFalse(parser._actions[-2].required)
        # arch is optional, and last param
        self.assertEqual(parser._actions[-1].dest, 'arch')
        self.assertFalse(parser._actions[-1].required)

    def test_take_action(self):
        response = Response()
        response.status_code = 200
        response._content = json.dumps({'successful': True}).encode(
            'utf-8')
        self.mock_post.return_value = response

        name = 'click_toolbelt.upload.Upload.mark_uploaded'
        with patch(name) as mock_mark_uploaded:
            mock_mark_uploaded.return_value = {'success': True}
            self.command.take_action(self.args)

        # assert request was made to get upload data
        # followed by an updown upload request
        self.mock_get_upload_data.assert_called_once_with(
            self.app_name, self.app_version, self.app_changelog,
            self.app_archs, self.app_framework)
        self.mock_post.assert_called_with(
            CLICK_UPDOWN_UPLOAD_URL,
            files={'binary': self.mock_open.return_value},
            data=self.mock_get_upload_data.return_value)
        self.mock_log.info.assert_called_with(
            'Click Package upload finished successfully.')

    def test_take_action_handles_error_while_getting_upload_data(self):
        response = Response()
        response.status_code = 200
        self.mock_post.return_value = response
        self.mock_get_upload_data.return_value = {
            'success': False, 'errors': ['Some error']}

        self.command.take_action(self.args)

        self.mock_log.error.assert_called_with(
            'Error verifying upload request. Uploading not possible.\n'
            'Reason: %s', 'Some error')

    def test_take_action_handles_error_while_uploading(self):
        self.mock_post.side_effect = Exception('foo')

        self.command.take_action(self.args)

        self.mock_log.exception.assert_called_with(
            'There was an error uploading the click package.')

    def test_take_action_handles_unsuccessful_response(self):
        upload_response = {'success': True, 'errors': []}
        response = Response()
        response.status_code = 200
        response.reason = 'OK'
        response._content = json.dumps(upload_response).encode('utf-8')
        self.mock_post.return_value = response

        self.command.take_action(self.args)

        self.mock_log.error.assert_called_with(
            'There was an error uploading the click package.\n'
            'Reason: %s\n'
            'Text: %s',
            'OK', json.dumps(upload_response))

    def test_take_action_handles_error_response(self):
        response = Response()
        response.status_code = 500
        response.reason = 'INTERNAL SERVER ERROR'
        response._content = 'This is a traceback.'.encode('utf-8')
        self.mock_post.return_value = response

        self.command.take_action(self.args)

        self.mock_log.error.assert_called_with(
            'There was an error uploading the click package.\n'
            'Reason: %s\n'
            'Text: %s',
            'INTERNAL SERVER ERROR', 'This is a traceback.')

    def test_take_action_handles_unsuccessful_mark_uploaded_response(self):
        upload_response = {'successful': True, 'errors': []}
        response = Response()
        response.status_code = 200
        response.reason = 'OK'
        response._content = json.dumps(upload_response).encode('utf-8')
        self.mock_post.return_value = response

        name = 'click_toolbelt.upload.Upload.mark_uploaded'
        with patch(name) as mock_mark_uploaded:
            mock_mark_uploaded.return_value = {
                'success': False, 'errors': ['Please try again later']}
            self.command.take_action(self.args)

        self.mock_log.error.assert_called_with(
            'There was an error after uploading the click package.\n'
            'Reason: %s', 'Please try again later')
